import { ChangeDetectorRef, Component, Directive, EventEmitter, Injectable, Input, NgModule, Output, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
var AngularPaginatorService = (function () {
    function AngularPaginatorService() {
        this.id = 'ANGULAR_PAGINATOR_DEFAULT';
        this.instances = {};
        this.collection = [];
        this.change = new EventEmitter();
    }
    /**
     * @param {?} instance
     * @return {?}
     */
    AngularPaginatorService.prototype.registerInstance = function (instance) {
        if (!this.instances[instance.id]) {
            this.instances[instance.id] = instance;
            this.change.emit(instance.id);
        }
        else {
            var /** @type {?} */ changed = this.updateInstance(instance);
            if (changed) {
                this.change.emit(instance.id);
            }
        }
    };
    /**
     * @param {?} instance
     * @return {?}
     */
    AngularPaginatorService.prototype.updateInstance = function (instance) {
        for (var /** @type {?} */ key in instance) {
            if (instance[key] !== this.instances[instance['id']][key]) {
                this.instances[instance['id']][key] = instance[key];
                return true;
            }
        }
        return false;
    };
    /**
     * @param {?} id
     * @return {?}
     */
    AngularPaginatorService.prototype.getInstance = function (id) {
        return this.instances[id];
    };
    return AngularPaginatorService;
}());
AngularPaginatorService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AngularPaginatorService.ctorParameters = function () { return []; };
var AngularPaginatorComponent = (function () {
    /**
     * @param {?} _angularPaginatorService
     */
    function AngularPaginatorComponent(_angularPaginatorService) {
        this._angularPaginatorService = _angularPaginatorService;
        this.boundaryLinks = false;
        this.directionLinks = true;
        this.maxSize = 5;
        this.rotate = true;
        this.boundaryLinkNumbers = false;
        this.forceEllipses = false;
        this.firstText = 'First';
        this.previousText = 'Previous';
        this.nextText = 'Next';
        this.lastText = 'Last';
        this.pageChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    AngularPaginatorComponent.prototype.setCustomTemplate = function () {
    };
    /**
     * @return {?}
     */
    AngularPaginatorComponent.prototype.ngOnInit = function () {
        // set default id if not provided
        this.id = this.id ? this.id : this._angularPaginatorService.id;
        // set default class if not provided
        this.class = this.class ? this.class : this._angularPaginatorService.id;
    };
    return AngularPaginatorComponent;
}());
AngularPaginatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-angular-paginator',
                template: "\n    <appAngularPaginator [id]=\"id\" [size]=\"size\" [boundaryLinks]=\"boundaryLinks\" [directionLinks]=\"directionLinks\" [maxSize]=\"maxSize\"\n      [rotate]=\"rotate\" [boundaryLinkNumbers]=\"boundaryLinkNumbers\" [forceEllipses]=\"forceEllipses\" [class]=\"class\" #paginator=\"angularPaginator\"\n      (pageChange)=\"pageChange.emit($event)\">\n\n      <!-- ======================================== BOOTSTRAP 4 TEMPLATE ======================================== -->\n      <nav class=\"angular-bs4-pagination\">\n        <ul class=\"pagination\" [ngClass]=\"[paginator.getPaginationSize(), paginator.class]\" *ngIf=\"paginator.pages['length'] !== 0\">\n\n          <!-- boundary link -->\n          <li class=\"page-item\" *ngIf=\"boundaryLinks\" (click)=\"paginator.toFirstPage()\" [ngClass]=\"{'disabled': paginator.currentPage === paginator.firstPage}\">\n            <a class=\"page-link\" href=\"#\" [innerHTML]=\"firstText\"></a>\n          </li>\n\n          <!-- direction link -->\n          <li class=\"page-item\" *ngIf=\"directionLinks\" (click)=\"paginator.toPreviousPage()\" [ngClass]=\"{'disabled': paginator.currentPage === paginator.firstPage}\">\n            <a class=\"page-link\" href=\"#\" [innerHTML]=\"previousText\"></a>\n          </li>\n\n          <!-- pagination array -->\n          <li class=\"page-item\" *ngFor=\"let page of paginator.pages\" [ngClass]=\"{'active': page['active']}\" (click)=\"paginator.setCurrentPage(page['number'])\">\n            <a class=\"page-link\" href=\"#\">{{page['text']}}</a>\n          </li>\n\n          <!-- direction link -->\n          <li class=\"page-item\" *ngIf=\"directionLinks\" (click)=\"paginator.toNextPage()\" [ngClass]=\"{'disabled': paginator.currentPage === paginator.lastPage}\">\n            <a class=\"page-link\" href=\"#\" [innerHTML]=\"nextText\"></a>\n          </li>\n\n          <!-- boundary link -->\n          <li class=\"page-item\" *ngIf=\"boundaryLinks\" (click)=\"paginator.toLastPage()\" [ngClass]=\"{'disabled': paginator.currentPage === paginator.lastPage}\">\n            <a class=\"page-link\" href=\"#\" [innerHTML]=\"lastText\"></a>\n          </li>\n\n        </ul>\n      </nav>\n\n    </appAngularPaginator>\n  ",
                styles: ["\n    .angular-bs4-pagination .disabled {\n      pointer-events: none; }\n  "]
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorComponent.ctorParameters = function () { return [
    { type: AngularPaginatorService, },
]; };
AngularPaginatorComponent.propDecorators = {
    'boundaryLinks': [{ type: Input },],
    'directionLinks': [{ type: Input },],
    'maxSize': [{ type: Input },],
    'rotate': [{ type: Input },],
    'boundaryLinkNumbers': [{ type: Input },],
    'forceEllipses': [{ type: Input },],
    'size': [{ type: Input },],
    'id': [{ type: Input },],
    'firstText': [{ type: Input },],
    'previousText': [{ type: Input },],
    'nextText': [{ type: Input },],
    'lastText': [{ type: Input },],
    'class': [{ type: Input },],
    'pageChange': [{ type: Output },],
};
var AngularPaginatorPipe = (function () {
    /**
     * @param {?} _angularPaginatorService
     */
    function AngularPaginatorPipe(_angularPaginatorService) {
        this._angularPaginatorService = _angularPaginatorService;
    }
    /**
     * @param {?} array
     * @param {?=} args
     * @return {?}
     */
    AngularPaginatorPipe.prototype.transform = function (array, args) {
        var /** @type {?} */ instance = this.createInstance(array, args);
        // create pagination information
        this._angularPaginatorService.registerInstance(instance);
        // set the slicing range
        var /** @type {?} */ start = (args.currentPage - 1) * args.itemsPerPage;
        var /** @type {?} */ end = args.currentPage * args.itemsPerPage;
        return array.slice(start, end);
    };
    /**
     * @param {?} array
     * @param {?} args
     * @return {?}
     */
    AngularPaginatorPipe.prototype.createInstance = function (array, args) {
        return {
            id: args['id'] ? args.id : this._angularPaginatorService.id,
            currentPage: args['currentPage'] ? args['currentPage'] : 1,
            itemsPerPage: args['itemsPerPage'] ? args['itemsPerPage'] : 10,
            totalItems: array['length']
        };
    };
    return AngularPaginatorPipe;
}());
AngularPaginatorPipe.decorators = [
    { type: Pipe, args: [{
                name: 'angularPaginator'
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorPipe.ctorParameters = function () { return [
    { type: AngularPaginatorService, },
]; };
var AngularPaginatorDirective = (function () {
    /**
     * @param {?} _angularPaginatorService
     * @param {?} changeDetectorRef
     */
    function AngularPaginatorDirective(_angularPaginatorService, changeDetectorRef) {
        var _this = this;
        this._angularPaginatorService = _angularPaginatorService;
        this.changeDetectorRef = changeDetectorRef;
        this.firstPage = 1;
        this.pages = [];
        this.pageChange = new EventEmitter();
        // subscribe to changes
        this.subscription = this._angularPaginatorService.change.subscribe(function (id) {
            if (id === _this.id) {
                _this.updatePages();
            }
        });
    }
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.getPaginationSize = function () {
        if (this.size === 'sm' || this.size === 'lg') {
            return 'pagination-' + this.size;
        }
        return undefined;
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.toPreviousPage = function () {
        if (this.currentPage > this.firstPage) {
            this.setCurrentPage(this.currentPage - 1);
        }
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.toNextPage = function () {
        if (this.currentPage < this.lastPage) {
            this.setCurrentPage(this.currentPage + 1);
        }
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.toFirstPage = function () {
        this.setCurrentPage(this.firstPage);
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.toLastPage = function () {
        this.setCurrentPage(this.lastPage);
    };
    /**
     * @param {?} page
     * @return {?}
     */
    AngularPaginatorDirective.prototype.setCurrentPage = function (page) {
        if (this.currentPage !== page) {
            this.currentPage = page;
            this.pageChange.emit(page);
        }
    };
    /**
     * @param {?} number
     * @param {?} text
     * @param {?} isActive
     * @return {?}
     */
    AngularPaginatorDirective.prototype.makePage = function (number, text, isActive) {
        return {
            number: number,
            text: text,
            active: isActive
        };
    };
    /**
     * @param {?} currentPage
     * @param {?} itemsPerPage
     * @param {?} totalItems
     * @return {?}
     */
    AngularPaginatorDirective.prototype.getPages = function (currentPage, itemsPerPage, totalItems) {
        var /** @type {?} */ pages = [];
        // Default page limits
        var /** @type {?} */ totalPages = this.lastPage = Math.ceil(totalItems / itemsPerPage);
        var /** @type {?} */ startPage = 1;
        var /** @type {?} */ endPage = totalPages;
        var /** @type {?} */ isMaxSized = this.maxSize && this.maxSize < totalPages;
        // recompute if maxSize
        if (isMaxSized) {
            if (this.rotate) {
                // current page is displayed in the middle of the visible ones
                startPage = Math.max(currentPage - Math.floor(this.maxSize / 2), 1);
                endPage = startPage + this.maxSize - 1;
                // Adjust if limit is exceeded
                if (endPage > totalPages) {
                    endPage = totalPages;
                    startPage = endPage - this.maxSize + 1;
                }
            }
            else {
                // Visible pages are paginated with maxSize
                startPage = (Math.ceil(currentPage / this.maxSize) - 1) * this.maxSize + 1;
                // adjust last page if limit is exceeded
                endPage = Math.min(startPage + this.maxSize - 1, totalPages);
            }
        }
        // add page number links
        for (var /** @type {?} */ number = startPage; number <= endPage; number++) {
            var /** @type {?} */ page = this.makePage(number, number, number === currentPage);
            pages.push(page);
        }
        // add links to move between page sets
        if (isMaxSized && this.maxSize > 0 && (!this.rotate || this.forceEllipses || this.boundaryLinkNumbers)) {
            if (startPage > 1) {
                if (!this.boundaryLinkNumbers || startPage > 3) {
                    var /** @type {?} */ previousPageSet = this.makePage(startPage - 1, '...', false);
                    pages.unshift(previousPageSet);
                }
                if (this.boundaryLinkNumbers) {
                    if (startPage === 3) {
                        var /** @type {?} */ secondPageLink = this.makePage(2, '2', false);
                        pages.unshift(secondPageLink);
                    }
                    // add the first page
                    var /** @type {?} */ firstPageLink = this.makePage(1, '1', false);
                    pages.unshift(firstPageLink);
                }
            }
            if (endPage < totalPages) {
                if (!this.boundaryLinkNumbers || endPage < totalPages - 2) {
                    var /** @type {?} */ nextPageSet = this.makePage(endPage + 1, '...', false);
                    pages.push(nextPageSet);
                }
                if (this.boundaryLinkNumbers) {
                    if (endPage === totalPages - 2) {
                        var /** @type {?} */ secondToLastPageLink = this.makePage(totalPages - 1, totalPages - 1, false);
                        pages.push(secondToLastPageLink);
                    }
                    // add the last page
                    var /** @type {?} */ lastPageLink = this.makePage(totalPages, totalPages, false);
                    pages.push(lastPageLink);
                }
            }
        }
        return pages;
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.updatePages = function () {
        var _this = this;
        var /** @type {?} */ instance = this._angularPaginatorService.getInstance(this.id);
        var /** @type {?} */ correctedCurrentPage = this.outOfBoundCorrection(instance);
        if (correctedCurrentPage !== instance['currentPae']) {
            setTimeout(function () {
                _this.setCurrentPage(correctedCurrentPage);
                _this.pages = _this.getPages(instance.currentPage, instance.itemsPerPage, instance.totalItems);
                // detect for changes since OnPush Change detection stragtegy is used
                _this.changeDetectorRef.markForCheck();
            });
        }
        else {
            this.pages = this.getPages(instance.currentPage, instance.itemsPerPage, instance.totalItems);
        }
    };
    /**
     * @param {?} instance
     * @return {?}
     */
    AngularPaginatorDirective.prototype.outOfBoundCorrection = function (instance) {
        var /** @type {?} */ totalPages = Math.ceil(instance['totalItems'] / instance['itemsPerPage']);
        if (totalPages < instance['currentPage'] && 0 < totalPages) {
            return totalPages;
        }
        else if (instance['currentPage'] < 1) {
            return 1;
        }
        return instance['currentPage'];
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.isValidId = function () {
        if (!this._angularPaginatorService.getInstance(this.id)) {
            throw new Error('There is no instance registered with id `' + this.id + '`');
        }
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.ngOnInit = function () {
        this.isValidId();
        this.updatePages();
    };
    /**
     * @return {?}
     */
    AngularPaginatorDirective.prototype.ngOnDestroy = function () {
        // destroy the subscription when the directive is destroyed
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    return AngularPaginatorDirective;
}());
AngularPaginatorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'appAngularPaginator, [appAngularPaginator]',
                exportAs: 'angularPaginator'
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorDirective.ctorParameters = function () { return [
    { type: AngularPaginatorService, },
    { type: ChangeDetectorRef, },
]; };
AngularPaginatorDirective.propDecorators = {
    'boundaryLinks': [{ type: Input },],
    'directionLinks': [{ type: Input },],
    'maxSize': [{ type: Input },],
    'rotate': [{ type: Input },],
    'boundaryLinkNumbers': [{ type: Input },],
    'forceEllipses': [{ type: Input },],
    'size': [{ type: Input },],
    'id': [{ type: Input },],
    'class': [{ type: Input },],
    'pageChange': [{ type: Output },],
};
var AngularPaginatorModule = (function () {
    function AngularPaginatorModule() {
    }
    return AngularPaginatorModule;
}());
AngularPaginatorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                providers: [AngularPaginatorService],
                declarations: [AngularPaginatorComponent, AngularPaginatorPipe, AngularPaginatorDirective],
                exports: [AngularPaginatorComponent, AngularPaginatorPipe, AngularPaginatorDirective]
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorModule.ctorParameters = function () { return []; };
/**
 * Generated bundle index. Do not edit.
 */
export { AngularPaginatorModule, AngularPaginatorComponent as ɵb, AngularPaginatorDirective as ɵd, AngularPaginatorPipe as ɵc, AngularPaginatorService as ɵa };
//# sourceMappingURL=angular-paginator.es5.js.map
