import { ChangeDetectorRef, Component, Directive, EventEmitter, Injectable, Input, NgModule, Output, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';

class AngularPaginatorService {
    constructor() {
        this.id = 'ANGULAR_PAGINATOR_DEFAULT';
        this.instances = {};
        this.collection = [];
        this.change = new EventEmitter();
    }
    /**
     * @param {?} instance
     * @return {?}
     */
    registerInstance(instance) {
        if (!this.instances[instance.id]) {
            this.instances[instance.id] = instance;
            this.change.emit(instance.id);
        }
        else {
            const /** @type {?} */ changed = this.updateInstance(instance);
            if (changed) {
                this.change.emit(instance.id);
            }
        }
    }
    /**
     * @param {?} instance
     * @return {?}
     */
    updateInstance(instance) {
        for (const /** @type {?} */ key in instance) {
            if (instance[key] !== this.instances[instance['id']][key]) {
                this.instances[instance['id']][key] = instance[key];
                return true;
            }
        }
        return false;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    getInstance(id) {
        return this.instances[id];
    }
}
AngularPaginatorService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AngularPaginatorService.ctorParameters = () => [];

class AngularPaginatorComponent {
    /**
     * @param {?} _angularPaginatorService
     */
    constructor(_angularPaginatorService) {
        this._angularPaginatorService = _angularPaginatorService;
        this.boundaryLinks = false;
        this.directionLinks = true;
        this.maxSize = 5;
        this.rotate = true;
        this.boundaryLinkNumbers = false;
        this.forceEllipses = false;
        this.firstText = 'First';
        this.previousText = 'Previous';
        this.nextText = 'Next';
        this.lastText = 'Last';
        this.pageChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    setCustomTemplate() {
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // set default id if not provided
        this.id = this.id ? this.id : this._angularPaginatorService.id;
        // set default class if not provided
        this.class = this.class ? this.class : this._angularPaginatorService.id;
    }
}
AngularPaginatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-angular-paginator',
                template: `
    <appAngularPaginator [id]="id" [size]="size" [boundaryLinks]="boundaryLinks" [directionLinks]="directionLinks" [maxSize]="maxSize"
      [rotate]="rotate" [boundaryLinkNumbers]="boundaryLinkNumbers" [forceEllipses]="forceEllipses" [class]="class" #paginator="angularPaginator"
      (pageChange)="pageChange.emit($event)">

      <!-- ======================================== BOOTSTRAP 4 TEMPLATE ======================================== -->
      <nav class="angular-bs4-pagination">
        <ul class="pagination" [ngClass]="[paginator.getPaginationSize(), paginator.class]" *ngIf="paginator.pages['length'] !== 0">

          <!-- boundary link -->
          <li class="page-item" *ngIf="boundaryLinks" (click)="paginator.toFirstPage()" [ngClass]="{'disabled': paginator.currentPage === paginator.firstPage}">
            <a class="page-link" href="#" [innerHTML]="firstText"></a>
          </li>

          <!-- direction link -->
          <li class="page-item" *ngIf="directionLinks" (click)="paginator.toPreviousPage()" [ngClass]="{'disabled': paginator.currentPage === paginator.firstPage}">
            <a class="page-link" href="#" [innerHTML]="previousText"></a>
          </li>

          <!-- pagination array -->
          <li class="page-item" *ngFor="let page of paginator.pages" [ngClass]="{'active': page['active']}" (click)="paginator.setCurrentPage(page['number'])">
            <a class="page-link" href="#">{{page['text']}}</a>
          </li>

          <!-- direction link -->
          <li class="page-item" *ngIf="directionLinks" (click)="paginator.toNextPage()" [ngClass]="{'disabled': paginator.currentPage === paginator.lastPage}">
            <a class="page-link" href="#" [innerHTML]="nextText"></a>
          </li>

          <!-- boundary link -->
          <li class="page-item" *ngIf="boundaryLinks" (click)="paginator.toLastPage()" [ngClass]="{'disabled': paginator.currentPage === paginator.lastPage}">
            <a class="page-link" href="#" [innerHTML]="lastText"></a>
          </li>

        </ul>
      </nav>

    </appAngularPaginator>
  `,
                styles: [`
    .angular-bs4-pagination .disabled {
      pointer-events: none; }
  `]
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorComponent.ctorParameters = () => [
    { type: AngularPaginatorService, },
];
AngularPaginatorComponent.propDecorators = {
    'boundaryLinks': [{ type: Input },],
    'directionLinks': [{ type: Input },],
    'maxSize': [{ type: Input },],
    'rotate': [{ type: Input },],
    'boundaryLinkNumbers': [{ type: Input },],
    'forceEllipses': [{ type: Input },],
    'size': [{ type: Input },],
    'id': [{ type: Input },],
    'firstText': [{ type: Input },],
    'previousText': [{ type: Input },],
    'nextText': [{ type: Input },],
    'lastText': [{ type: Input },],
    'class': [{ type: Input },],
    'pageChange': [{ type: Output },],
};

class AngularPaginatorPipe {
    /**
     * @param {?} _angularPaginatorService
     */
    constructor(_angularPaginatorService) {
        this._angularPaginatorService = _angularPaginatorService;
    }
    /**
     * @param {?} array
     * @param {?=} args
     * @return {?}
     */
    transform(array, args) {
        const /** @type {?} */ instance = this.createInstance(array, args);
        // create pagination information
        this._angularPaginatorService.registerInstance(instance);
        // set the slicing range
        const /** @type {?} */ start = (args.currentPage - 1) * args.itemsPerPage;
        const /** @type {?} */ end = args.currentPage * args.itemsPerPage;
        return array.slice(start, end);
    }
    /**
     * @param {?} array
     * @param {?} args
     * @return {?}
     */
    createInstance(array, args) {
        return {
            id: args['id'] ? args.id : this._angularPaginatorService.id,
            currentPage: args['currentPage'] ? args['currentPage'] : 1,
            itemsPerPage: args['itemsPerPage'] ? args['itemsPerPage'] : 10,
            totalItems: array['length']
        };
    }
}
AngularPaginatorPipe.decorators = [
    { type: Pipe, args: [{
                name: 'angularPaginator'
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorPipe.ctorParameters = () => [
    { type: AngularPaginatorService, },
];

class AngularPaginatorDirective {
    /**
     * @param {?} _angularPaginatorService
     * @param {?} changeDetectorRef
     */
    constructor(_angularPaginatorService, changeDetectorRef) {
        this._angularPaginatorService = _angularPaginatorService;
        this.changeDetectorRef = changeDetectorRef;
        this.firstPage = 1;
        this.pages = [];
        this.pageChange = new EventEmitter();
        // subscribe to changes
        this.subscription = this._angularPaginatorService.change.subscribe(id => {
            if (id === this.id) {
                this.updatePages();
            }
        });
    }
    /**
     * @return {?}
     */
    getPaginationSize() {
        if (this.size === 'sm' || this.size === 'lg') {
            return 'pagination-' + this.size;
        }
        return undefined;
    }
    /**
     * @return {?}
     */
    toPreviousPage() {
        if (this.currentPage > this.firstPage) {
            this.setCurrentPage(this.currentPage - 1);
        }
    }
    /**
     * @return {?}
     */
    toNextPage() {
        if (this.currentPage < this.lastPage) {
            this.setCurrentPage(this.currentPage + 1);
        }
    }
    /**
     * @return {?}
     */
    toFirstPage() {
        this.setCurrentPage(this.firstPage);
    }
    /**
     * @return {?}
     */
    toLastPage() {
        this.setCurrentPage(this.lastPage);
    }
    /**
     * @param {?} page
     * @return {?}
     */
    setCurrentPage(page) {
        if (this.currentPage !== page) {
            this.currentPage = page;
            this.pageChange.emit(page);
        }
    }
    /**
     * @param {?} number
     * @param {?} text
     * @param {?} isActive
     * @return {?}
     */
    makePage(number, text, isActive) {
        return {
            number: number,
            text: text,
            active: isActive
        };
    }
    /**
     * @param {?} currentPage
     * @param {?} itemsPerPage
     * @param {?} totalItems
     * @return {?}
     */
    getPages(currentPage, itemsPerPage, totalItems) {
        const /** @type {?} */ pages = [];
        // Default page limits
        const /** @type {?} */ totalPages = this.lastPage = Math.ceil(totalItems / itemsPerPage);
        let /** @type {?} */ startPage = 1;
        let /** @type {?} */ endPage = totalPages;
        const /** @type {?} */ isMaxSized = this.maxSize && this.maxSize < totalPages;
        // recompute if maxSize
        if (isMaxSized) {
            if (this.rotate) {
                // current page is displayed in the middle of the visible ones
                startPage = Math.max(currentPage - Math.floor(this.maxSize / 2), 1);
                endPage = startPage + this.maxSize - 1;
                // Adjust if limit is exceeded
                if (endPage > totalPages) {
                    endPage = totalPages;
                    startPage = endPage - this.maxSize + 1;
                }
            }
            else {
                // Visible pages are paginated with maxSize
                startPage = (Math.ceil(currentPage / this.maxSize) - 1) * this.maxSize + 1;
                // adjust last page if limit is exceeded
                endPage = Math.min(startPage + this.maxSize - 1, totalPages);
            }
        }
        // add page number links
        for (let /** @type {?} */ number = startPage; number <= endPage; number++) {
            const /** @type {?} */ page = this.makePage(number, number, number === currentPage);
            pages.push(page);
        }
        // add links to move between page sets
        if (isMaxSized && this.maxSize > 0 && (!this.rotate || this.forceEllipses || this.boundaryLinkNumbers)) {
            if (startPage > 1) {
                if (!this.boundaryLinkNumbers || startPage > 3) {
                    const /** @type {?} */ previousPageSet = this.makePage(startPage - 1, '...', false);
                    pages.unshift(previousPageSet);
                }
                if (this.boundaryLinkNumbers) {
                    if (startPage === 3) {
                        const /** @type {?} */ secondPageLink = this.makePage(2, '2', false);
                        pages.unshift(secondPageLink);
                    }
                    // add the first page
                    const /** @type {?} */ firstPageLink = this.makePage(1, '1', false);
                    pages.unshift(firstPageLink);
                }
            }
            if (endPage < totalPages) {
                if (!this.boundaryLinkNumbers || endPage < totalPages - 2) {
                    const /** @type {?} */ nextPageSet = this.makePage(endPage + 1, '...', false);
                    pages.push(nextPageSet);
                }
                if (this.boundaryLinkNumbers) {
                    if (endPage === totalPages - 2) {
                        const /** @type {?} */ secondToLastPageLink = this.makePage(totalPages - 1, totalPages - 1, false);
                        pages.push(secondToLastPageLink);
                    }
                    // add the last page
                    const /** @type {?} */ lastPageLink = this.makePage(totalPages, totalPages, false);
                    pages.push(lastPageLink);
                }
            }
        }
        return pages;
    }
    /**
     * @return {?}
     */
    updatePages() {
        const /** @type {?} */ instance = this._angularPaginatorService.getInstance(this.id);
        const /** @type {?} */ correctedCurrentPage = this.outOfBoundCorrection(instance);
        if (correctedCurrentPage !== instance['currentPae']) {
            setTimeout(() => {
                this.setCurrentPage(correctedCurrentPage);
                this.pages = this.getPages(instance.currentPage, instance.itemsPerPage, instance.totalItems);
                // detect for changes since OnPush Change detection stragtegy is used
                this.changeDetectorRef.markForCheck();
            });
        }
        else {
            this.pages = this.getPages(instance.currentPage, instance.itemsPerPage, instance.totalItems);
        }
    }
    /**
     * @param {?} instance
     * @return {?}
     */
    outOfBoundCorrection(instance) {
        const /** @type {?} */ totalPages = Math.ceil(instance['totalItems'] / instance['itemsPerPage']);
        if (totalPages < instance['currentPage'] && 0 < totalPages) {
            return totalPages;
        }
        else if (instance['currentPage'] < 1) {
            return 1;
        }
        return instance['currentPage'];
    }
    /**
     * @return {?}
     */
    isValidId() {
        if (!this._angularPaginatorService.getInstance(this.id)) {
            throw new Error('There is no instance registered with id `' + this.id + '`');
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.isValidId();
        this.updatePages();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // destroy the subscription when the directive is destroyed
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
AngularPaginatorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'appAngularPaginator, [appAngularPaginator]',
                exportAs: 'angularPaginator'
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorDirective.ctorParameters = () => [
    { type: AngularPaginatorService, },
    { type: ChangeDetectorRef, },
];
AngularPaginatorDirective.propDecorators = {
    'boundaryLinks': [{ type: Input },],
    'directionLinks': [{ type: Input },],
    'maxSize': [{ type: Input },],
    'rotate': [{ type: Input },],
    'boundaryLinkNumbers': [{ type: Input },],
    'forceEllipses': [{ type: Input },],
    'size': [{ type: Input },],
    'id': [{ type: Input },],
    'class': [{ type: Input },],
    'pageChange': [{ type: Output },],
};

class AngularPaginatorModule {
}
AngularPaginatorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                providers: [AngularPaginatorService],
                declarations: [AngularPaginatorComponent, AngularPaginatorPipe, AngularPaginatorDirective],
                exports: [AngularPaginatorComponent, AngularPaginatorPipe, AngularPaginatorDirective]
            },] },
];
/**
 * @nocollapse
 */
AngularPaginatorModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { AngularPaginatorModule, AngularPaginatorComponent as ɵb, AngularPaginatorDirective as ɵd, AngularPaginatorPipe as ɵc, AngularPaginatorService as ɵa };
//# sourceMappingURL=angular-paginator.js.map
